"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiskFS = exports.InMemoryFS = exports.Dir = exports.createModuleResolutionHost = exports.joinPaths = exports.hasWindowsSlashes = exports.normalizeSlashes = void 0;
const assert_1 = __importDefault(require("assert"));
const path_1 = require("path");
const assertions_1 = require("./assertions");
const fs_extra_1 = require("fs-extra");
const io_1 = require("./io");
/** Convert a path to use "/" instead of "\\" for consistency. (This affects content hash.) */
function normalizeSlashes(path) {
    return path.replace(/\\/g, "/");
}
exports.normalizeSlashes = normalizeSlashes;
function hasWindowsSlashes(path) {
    return path.includes("\\");
}
exports.hasWindowsSlashes = hasWindowsSlashes;
/** Always use "/" for consistency. (This affects package content hash.) */
function joinPaths(...paths) {
    return paths.join("/");
}
exports.joinPaths = joinPaths;
function createModuleResolutionHost(fs, ignoreFilesAboveDirectory) {
    return {
        fileExists: (fileName) => !isIgnored(fileName) && fs.exists(fileName),
        readFile: (fileName) => ((0, assert_1.default)(!isIgnored(fileName)), fs.readFile(fileName)),
        directoryExists: (directoryName) => !isIgnored(directoryName) && fs.exists(directoryName),
        getCurrentDirectory: () => "",
        realpath: (path) => path,
        useCaseSensitiveFileNames: () => true,
    };
    function isIgnored(path) {
        return (ignoreFilesAboveDirectory !== undefined &&
            path !== ignoreFilesAboveDirectory &&
            !path.startsWith(ignoreFilesAboveDirectory));
    }
}
exports.createModuleResolutionHost = createModuleResolutionHost;
// Map entries are Dir for directory and string for file.
class Dir extends Map {
    constructor(parent) {
        super();
        this.parent = parent;
    }
    subdir(name) {
        const x = this.get(name);
        if (x !== undefined) {
            if (typeof x === "string") {
                throw new Error(`File ${name} has same name as a directory?`);
            }
            return x;
        }
        const res = new Dir(this);
        this.set(name, res);
        return res;
    }
    finish() {
        const out = new Dir(this.parent);
        for (const key of Array.from(this.keys()).sort()) {
            const subDirOrFile = this.get(key);
            out.set(key, typeof subDirOrFile === "string" ? subDirOrFile : subDirOrFile.finish());
        }
        return out;
    }
}
exports.Dir = Dir;
function ensureTrailingSlash(dir) {
    return dir.endsWith("/") ? dir : dir + "/";
}
class InMemoryFS {
    constructor(curDir, rootPrefix) {
        this.curDir = curDir;
        this.rootPrefix = rootPrefix;
        this.rootPrefix = ensureTrailingSlash(rootPrefix);
        (0, assert_1.default)(rootPrefix[0] === "/", `rootPrefix must be absolute: ${rootPrefix}`);
    }
    tryGetEntry(path) {
        if (path[0] === "/") {
            path = (0, path_1.relative)(this.rootPrefix, path);
        }
        if (path === "") {
            return this.curDir;
        }
        const needsDir = path.endsWith("/");
        if (needsDir) {
            path = path.slice(0, -1);
        }
        const components = path.split("/");
        const baseName = (0, assertions_1.assertDefined)(components.pop());
        let dir = this.curDir;
        for (const component of components) {
            const entry = component === ".." ? dir.parent : dir.get(component);
            if (entry === undefined) {
                return undefined;
            }
            if (!(entry instanceof Dir)) {
                throw new Error(`No file system entry at ${this.rootPrefix}/${path}. Siblings are: ${Array.from(dir.keys()).toString()}`);
            }
            dir = entry;
        }
        const res = dir.get(baseName);
        return needsDir ? (res instanceof Dir ? res : undefined) : res;
    }
    getEntry(path) {
        const entry = this.tryGetEntry(path);
        if (entry === undefined) {
            throw new Error(`No file system entry at ${this.rootPrefix}/${path}`);
        }
        return entry;
    }
    getDir(dirPath) {
        const res = this.getEntry(dirPath);
        if (!(res instanceof Dir)) {
            throw new Error(`${this.rootPrefix}/${dirPath} is a file, not a directory.`);
        }
        return res;
    }
    readFile(filePath) {
        const res = this.getEntry(filePath);
        if (typeof res !== "string") {
            throw new Error(`${this.rootPrefix}/${filePath} is a directory, not a file.`);
        }
        return res;
    }
    readdir(dirPath) {
        return Array.from((dirPath === undefined ? this.curDir : this.getDir(dirPath)).keys());
    }
    readJson(path) {
        return JSON.parse(this.readFile(path));
    }
    isDirectory(path) {
        return typeof this.getEntry(path) !== "string";
    }
    exists(path) {
        return this.tryGetEntry(path) !== undefined;
    }
    subDir(path) {
        (0, assert_1.default)(path[0] !== "/", "Cannot use absolute paths with InMemoryFS.subDir");
        return new InMemoryFS(this.getDir(path), (0, path_1.resolve)(this.rootPrefix, path));
    }
    debugPath() {
        return this.rootPrefix;
    }
}
exports.InMemoryFS = InMemoryFS;
class DiskFS {
    constructor(rootPrefix) {
        this.rootPrefix = rootPrefix;
        (0, assert_1.default)(rootPrefix.startsWith("/"), "DiskFS must use absolute paths");
        this.rootPrefix = ensureTrailingSlash(rootPrefix);
    }
    getPath(path) {
        return (0, path_1.resolve)(this.rootPrefix, path !== null && path !== void 0 ? path : "");
    }
    readdir(dirPath) {
        return (0, fs_extra_1.readdirSync)(this.getPath(dirPath))
            .sort()
            .filter((name) => name !== ".DS_Store");
    }
    isDirectory(dirPath) {
        return (0, fs_extra_1.statSync)(this.getPath(dirPath)).isDirectory();
    }
    readJson(path) {
        return (0, io_1.readJsonSync)(this.getPath(path));
    }
    readFile(path) {
        return (0, io_1.readFileSync)(this.getPath(path));
    }
    exists(path) {
        return (0, fs_extra_1.pathExistsSync)(this.getPath(path));
    }
    subDir(path) {
        return new DiskFS(`${this.rootPrefix}${path}/`);
    }
    debugPath() {
        return this.rootPrefix.slice(0, this.rootPrefix.length - 1); // remove trailing '/'
    }
}
exports.DiskFS = DiskFS;
//# sourceMappingURL=fs.js.map